/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Test the Unicode character type functions.
   Copyright (C) 2007 Free Software Foundation, Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#include "test-predicate-part1.h"

    { 0x094D, 0x094D },
    { 0x09CD, 0x09CD },
    { 0x0A4D, 0x0A4D },
    { 0x0ACD, 0x0ACD },
    { 0x0B4D, 0x0B4D },
    { 0x0BCD, 0x0BCD },
    { 0x0C4D, 0x0C4D },
    { 0x0CCD, 0x0CCD },
    { 0x0D4D, 0x0D4D },
    { 0x0DCA, 0x0DCA },
    { 0x0E3A, 0x0E3A },
    { 0x0F84, 0x0F84 },
    { 0x1039, 0x103A },
    { 0x1714, 0x1714 },
    { 0x1734, 0x1734 },
    { 0x17D2, 0x17D2 },
    { 0x1A60, 0x1A60 },
    { 0x1B44, 0x1B44 },
    { 0x1BAA, 0x1BAA },
    { 0x1BF2, 0x1BF3 },
    { 0x2D7F, 0x2D7F },
    { 0xA806, 0xA806 },
    { 0xA8C4, 0xA8C4 },
    { 0xA953, 0xA953 },
    { 0xA9C0, 0xA9C0 },
    { 0xABED, 0xABED },
    { 0x10A3F, 0x10A3F },
    { 0x11046, 0x11046 },
    { 0x110B9, 0x110B9 }

#define PREDICATE(c) uc_is_property_grapheme_link (c)
#include "test-predicate-part2.h"
