/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Test the Unicode character type functions.
   Copyright (C) 2007 Free Software Foundation, Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#include "test-predicate-part1.h"

    { 0x0021, 0x0021 },
    { 0x002C, 0x002C },
    { 0x002E, 0x002E },
    { 0x003A, 0x003B },
    { 0x003F, 0x003F },
    { 0x037E, 0x037E },
    { 0x0387, 0x0387 },
    { 0x0589, 0x0589 },
    { 0x05C3, 0x05C3 },
    { 0x060C, 0x060C },
    { 0x061B, 0x061B },
    { 0x061F, 0x061F },
    { 0x06D4, 0x06D4 },
    { 0x0700, 0x070A },
    { 0x070C, 0x070C },
    { 0x07F8, 0x07F9 },
    { 0x0830, 0x083E },
    { 0x085E, 0x085E },
    { 0x0964, 0x0965 },
    { 0x0E5A, 0x0E5B },
    { 0x0F08, 0x0F08 },
    { 0x0F0D, 0x0F12 },
    { 0x104A, 0x104B },
    { 0x1361, 0x1368 },
    { 0x166D, 0x166E },
    { 0x16EB, 0x16ED },
    { 0x17D4, 0x17D6 },
    { 0x17DA, 0x17DA },
    { 0x1802, 0x1805 },
    { 0x1808, 0x1809 },
    { 0x1944, 0x1945 },
    { 0x1AA8, 0x1AAB },
    { 0x1B5A, 0x1B5B },
    { 0x1B5D, 0x1B5F },
    { 0x1C3B, 0x1C3F },
    { 0x1C7E, 0x1C7F },
    { 0x203C, 0x203D },
    { 0x2047, 0x2049 },
    { 0x2E2E, 0x2E2E },
    { 0x3001, 0x3002 },
    { 0xA4FE, 0xA4FF },
    { 0xA60D, 0xA60F },
    { 0xA6F3, 0xA6F7 },
    { 0xA876, 0xA877 },
    { 0xA8CE, 0xA8CF },
    { 0xA92F, 0xA92F },
    { 0xA9C7, 0xA9C9 },
    { 0xAA5D, 0xAA5F },
    { 0xAADF, 0xAADF },
    { 0xABEB, 0xABEB },
    { 0xFE50, 0xFE52 },
    { 0xFE54, 0xFE57 },
    { 0xFF01, 0xFF01 },
    { 0xFF0C, 0xFF0C },
    { 0xFF0E, 0xFF0E },
    { 0xFF1A, 0xFF1B },
    { 0xFF1F, 0xFF1F },
    { 0xFF61, 0xFF61 },
    { 0xFF64, 0xFF64 },
    { 0x1039F, 0x1039F },
    { 0x103D0, 0x103D0 },
    { 0x10857, 0x10857 },
    { 0x1091F, 0x1091F },
    { 0x10B3A, 0x10B3F },
    { 0x11047, 0x1104D },
    { 0x110BE, 0x110C1 },
    { 0x12470, 0x12473 }

#define PREDICATE(c) uc_is_property_terminal_punctuation (c)
#include "test-predicate-part2.h"
