/* Test of fma().
   Copyright (C) 2011-2014 Free Software Foundation, Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* Written by Bruno Haible <bruno@clisp.org>, 2011.  */

#include <config.h>

#include <math.h>

#include "float+.h"
#include "infinity.h"
#include "macros.h"

#undef INFINITY

#define DOUBLE double
#define LDEXP ldexp
const int MIN_EXP = DBL_MIN_EXP; /* for gdb */
#define MIN_EXP DBL_MIN_EXP
const int MAX_EXP = DBL_MAX_EXP; /* for gdb */
#define MAX_EXP DBL_MAX_EXP
const int MANT_BIT = DBL_MANT_BIT; /* for gdb */
#define MANT_BIT DBL_MANT_BIT
#define INFINITY Infinityd ()
#define L_(literal) literal
#include "test-fma2.h"

int
main ()
{
  test_function (fma);

  return 0;
}
