/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Properties of Unicode characters.  */
/* Generated automatically by gen-uni-tables.c for Unicode 6.0.0.  */
#define header_0 16
#define header_2 9
#define header_3 127
#define header_4 15
static const
struct
  {
    int header[1];
    int level1[3];
    short level2[3 << 7];
    /*unsigned*/ int level3[50 << 4];
  }
u_property_alphabetic =
{
  { 3 },
  {
        4 * sizeof (int) / sizeof (short) +     0,
        4 * sizeof (int) / sizeof (short) +   128,
        4 * sizeof (int) / sizeof (short) +   256
  },
  {
        4 +   384 * sizeof (short) / sizeof (int) +     0,
        4 +   384 * sizeof (short) / sizeof (int) +    16,
        4 +   384 * sizeof (short) / sizeof (int) +    32,
        4 +   384 * sizeof (short) / sizeof (int) +    48,
        4 +   384 * sizeof (short) / sizeof (int) +    64,
        4 +   384 * sizeof (short) / sizeof (int) +    80,
        4 +   384 * sizeof (short) / sizeof (int) +    96,
        4 +   384 * sizeof (short) / sizeof (int) +   112,
        4 +   384 * sizeof (short) / sizeof (int) +   128,
        4 +   384 * sizeof (short) / sizeof (int) +   144,
        4 +   384 * sizeof (short) / sizeof (int) +   160,
        4 +   384 * sizeof (short) / sizeof (int) +   176,
        4 +   384 * sizeof (short) / sizeof (int) +   192,
        4 +   384 * sizeof (short) / sizeof (int) +   208,
        4 +   384 * sizeof (short) / sizeof (int) +   224,
        4 +   384 * sizeof (short) / sizeof (int) +   240,
        4 +   384 * sizeof (short) / sizeof (int) +   256,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   272,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   288,
        4 +   384 * sizeof (short) / sizeof (int) +   304,
        4 +   384 * sizeof (short) / sizeof (int) +   320,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   352,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   368,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   384,
        4 +   384 * sizeof (short) / sizeof (int) +   400,
        4 +   384 * sizeof (short) / sizeof (int) +   416,
        4 +   384 * sizeof (short) / sizeof (int) +   432,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   448,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   464,
        4 +   384 * sizeof (short) / sizeof (int) +   480,
        4 +   384 * sizeof (short) / sizeof (int) +   496,
        4 +   384 * sizeof (short) / sizeof (int) +   512,
        4 +   384 * sizeof (short) / sizeof (int) +   528,
        4 +   384 * sizeof (short) / sizeof (int) +   544,
        4 +   384 * sizeof (short) / sizeof (int) +   560,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   576,
        4 +   384 * sizeof (short) / sizeof (int) +   592,
        4 +   384 * sizeof (short) / sizeof (int) +   608,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   624,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   640,
        4 +   384 * sizeof (short) / sizeof (int) +   656,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   672,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   688,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   704,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   720,
        4 +   384 * sizeof (short) / sizeof (int) +   736,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   752,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   768,
        4 +   384 * sizeof (short) / sizeof (int) +   784,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   784,
       -1,
       -1
  },
  {
    0x00000000, 0x00000000, 0x07FFFFFE, 0x07FFFFFE,
    0x00000000, 0x04200400, 0xFF7FFFFF, 0xFF7FFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x0003FFC3, 0x0000501F,
    0x00000000, 0x00000000, 0x00000020, 0x3CDF0000,
    0xFFFFD740, 0xFFFFFFFB, 0xFFFFFFFF, 0xFFBFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFC03, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFE00FF, 0x027FFFFF, 0xFFFFFFFE,
    0x000000FF, 0xBFFF0000, 0xFFFF00B6, 0x000707FF,
    0x07FF0000, 0xFFFFFFFF, 0xFEFFFFFF, 0xFFFFC000,
    0xFFFFFFFF, 0xFFFFFFFF, 0x1FEFFFFF, 0x9C00E1FE,
    0xFFFF0000, 0xFFFFFFFF, 0xFFFFE000, 0xFFFFFFFF,
    0xFFFFFFFF, 0x0003FFFF, 0xFFFFFC00, 0x043007FF,
    0xFCFFFFFF, 0x00001FFF, 0x01FFFFFF, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xEFFFFFFF, 0xFFE1DFFF, 0xFEFE000F,
    0xFFF99FEE, 0xE3C5FDFF, 0xB080599F, 0x0003000F,
    0xFFF987EE, 0xC36DFDFF, 0x5E021987, 0x003F0000,
    0xFFFBBFEE, 0xE3EDFDFF, 0x00011BBF, 0x0000000F,
    0xFFF99FEE, 0xE3EDFDFF, 0xB0C0199F, 0x0002000F,
    0xD63DC7EC, 0xC3FFC718, 0x00811DC7, 0x00000000,
    0xFFFDDFEE, 0xE3EFFDFF, 0x03601DDF, 0x0000000F,
    0xFFFDDFEC, 0xE3EFFDFF, 0x40601DDF, 0x0006000F,
    0xFFFDDFEC, 0xE7FFFFFF, 0x00805DDF, 0xFC00000F,
    0xFC7FFFEC, 0x2FFBFFFF, 0xFF5F807F, 0x000C0000,
    0xFFFFFFFE, 0x07FFFFFF, 0x0000207F, 0x00000000,
    0xFEF02596, 0x3BFFECAE, 0x3000205F, 0x00000000,
    0x00000001, 0x00000000, 0xFFFFFEFF, 0xFFFE1FFF,
    0xFEFFFF03, 0x1FFFFFFF, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xF97FFFFF, 0xFFFF0000, 0xFFFFC1E7,
    0x3000407F, 0xFFFFFFFF, 0xFFFF003F, 0x17FFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x3D7F3DFF, 0xFFFFFFFF,
    0xFFFF3DFF, 0x7F3DFFFF, 0xFF7FFF3D, 0xFFFFFFFF,
    0xFF3DFFFF, 0xFFFFFFFF, 0x87FFFFFF, 0x00000000,
    0x0000FFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x001FFFFF,
    0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFF9FFF,
    0x07FFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0x0001C7FF,
    0x000FDFFF, 0x000FFFFF, 0x000FFFFF, 0x000DDFFF,
    0xFFFFFFFF, 0xFFCFFFFF, 0x108001FF, 0x00000000,
    0x00000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00FFFFFF,
    0xFFFFFFFF, 0xFFFF07FF, 0xFFFFFFFF, 0x003FFFFF,
    0x1FFFFFFF, 0x01FF0FFF, 0xFFFF0000, 0x001F3FFF,
    0xFFFFFFFF, 0xFFFF0FFF, 0x000003FF, 0x00000000,
    0x0FFFFFFF, 0xFFFFFFFF, 0x7FFFFFFF, 0x001FFFFE,
    0x00000000, 0x00000080, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFEFFFFF, 0x00000FEF, 0x00000000,
    0xFFFFFFFF, 0x0000C3FF, 0xFFFFFFFF, 0x0003FFBF,
    0xFFFFFFFF, 0x003FFFFF, 0xFC00E000, 0x3FFFFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x0007DE00,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x3F3FFFFF, 0xFFFFFFFF, 0xAAFF3F3F, 0x3FFFFFFF,
    0xFFFFFFFF, 0x5FDFFFFF, 0x0FCF1FDC, 0x1FDC1FFF,
    0x00000000, 0x00000000, 0x00000000, 0x80020000,
    0x1FFF0000, 0x00000000, 0x00000000, 0x00000000,
    0x3E2FFC84, 0xF3FFBD50, 0x000043E0, 0xFFFFFFFF,
    0x000001FF, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0xFFC00000, 0xFFFFFFFF, 0x000003FF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFF7FFF, 0x7FFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0000781F,
    0xFFFFFFFF, 0xFFFF003F, 0xFFFFFFFF, 0x0000803F,
    0x007FFFFF, 0x7F7F7F7F, 0x7F7F7F7F, 0xFFFFFFFF,
    0x00000000, 0x00008000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x000000E0, 0x1F3E03FE, 0xFFFFFFFE, 0xFFFFFFFF,
    0xE07FFFFF, 0xFFFFFFFE, 0xFFFFFFFF, 0xF7FFFFFF,
    0xFFFFFFE0, 0xFFFE3FFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x00007FFF, 0x07FFFFFF, 0x00000000, 0xFFFF0000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x003FFFFF, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x00000FFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x00001FFF, 0x00000000, 0xFFFF0000, 0x3FFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFF1FFF, 0x00000C00, 0xFFFFFFFF, 0x80007FFF,
    0x00FFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0000FFFF,
    0xFF800000, 0xFFFFFFFC, 0xFFFFFFFF, 0xFFFFFFFF,
    0x000379FF, 0x000003FF, 0x00000000, 0xFC000000,
    0xFFFFF7BB, 0x000000FF, 0xFFFFFFFF, 0x000FFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x0000000F, 0x08FC0000,
    0xFFFFFC00, 0xFFFF07FF, 0x0007FFFF, 0x1FFFFFFF,
    0xFFFFFFFF, 0xFFF7FFFF, 0x00008000, 0x00000000,
    0xFFFFFFFF, 0x007FFFFF, 0x00003FFF, 0x047FFFFF,
    0xFFFFFFFF, 0x7FFFFFFF, 0x38000005, 0x00000000,
    0x007E7E7E, 0x00007F7F, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0xFFFFFFFF, 0x000007FF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFF000F, 0xFFFFF87F, 0x0FFFFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFF3FFF, 0xFFFFFFFF, 0xFFFF3FFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x03FFFFFF, 0x00000000,
    0xE0F8007F, 0x5F7FFDFF, 0xFFFFFFDB, 0xFFFFFFFF,
    0xFFFFFFFF, 0x0003FFFF, 0xFFF80000, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x3FFFFFFF, 0xFFFF0000, 0xFFFFFFFF,
    0xFFFCFFFF, 0xFFFFFFFF, 0x000000FF, 0x0FFF0000,
    0x00000000, 0x00000000, 0x00000000, 0xFFDF0000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x1FFFFFFF,
    0x00000000, 0x07FFFFFE, 0x07FFFFFE, 0xFFFFFFC0,
    0xFFFFFFFF, 0x7FFFFFFF, 0x1CFCFCFC, 0x00000000,
    0xFFFFEFFF, 0xB7FFFF7F, 0x3FFF3FFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x07FFFFFF,
    0x00000000, 0x00000000, 0xFFFFFFFF, 0x001FFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x1FFFFFFF, 0xFFFFFFFF, 0x0001FFFF, 0x00000000,
    0x7FFFFFFF, 0xFFFF0000, 0x000007FF, 0x00000000,
    0x3FFFFFFF, 0xFFFFFFFF, 0x003EFF0F, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x3FFFFFFF, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFD3F, 0x91BFFFFF, 0x003FFFFF, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x003FFFFF, 0x03FFFFFF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFEEFF06F, 0x000FFFFF, 0x00000000, 0x1FFFFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0x003FFFFF, 0x003FFFFF, 0x0007FFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0x000001FF, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0x0000003F, 0x00000000,
    0xFFFFFFFC, 0x01FFFFFF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00007FFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000007,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0x00007FFF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0x01FFFFFF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000003, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFDFFFFF, 0xFFFFFFFF,
    0xDFFFFFFF, 0xEBFFDE64, 0xFFFFFFEF, 0xFFFFFFFF,
    0xDFDFE7BF, 0x7BFFFFFF, 0xFFFDFC5F, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFF3F, 0xF7FFFFFD, 0xF7FFFFFF,
    0xFFDFFFFF, 0xFFDFFFFF, 0xFFFF7FFF, 0xFFFF7FFF,
    0xFFFFFDFF, 0xFFFFFDFF, 0x00000FF7, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x007FFFFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x001FFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x3FFFFFFF, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000
  }
};
