/* Arabic joining group of Unicode characters.
   Copyright (C) 2011-2014 Free Software Foundation, Inc.
   Written by Bruno Haible <bruno@clisp.org>, 2011.

   This program is free software: you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as published
   by the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

ELEM (NONE,                  "No Joining Group")
ELEM (AIN,                   "Ain")
ELEM (ALAPH,                 "Alaph")
ELEM (ALEF,                  "Alef")
ELEM (BEH,                   "Beh")
ELEM (BETH,                  "Beth")
ELEM (BURUSHASKI_YEH_BARREE, "Burushaski Yeh Barree")
ELEM (DAL,                   "Dal")
ELEM (DALATH_RISH,           "Dalath Rish")
ELEM (E,                     "E")
ELEM (FARSI_YEH,             "Farsi Yeh")
ELEM (FE,                    "Fe")
ELEM (FEH,                   "Feh")
ELEM (FINAL_SEMKATH,         "Final Semkath")
ELEM (GAF,                   "Gaf")
ELEM (GAMAL,                 "Gamal")
ELEM (HAH,                   "Hah")
ELEM (HE,                    "He")
ELEM (HEH,                   "Heh")
ELEM (HEH_GOAL,              "Heh Goal")
ELEM (HETH,                  "Heth")
ELEM (KAF,                   "Kaf")
ELEM (KAPH,                  "Kaph")
ELEM (KHAPH,                 "Khaph")
ELEM (KNOTTED_HEH,           "Knotted Heh")
ELEM (LAM,                   "Lam")
ELEM (LAMADH,                "Lamadh")
ELEM (MEEM,                  "Meem")
ELEM (MIM,                   "Mim")
ELEM (NOON,                  "Noon")
ELEM (NUN,                   "Nun")
ELEM (NYA,                   "Nya")
ELEM (PE,                    "Pe")
ELEM (QAF,                   "Qaf")
ELEM (QAPH,                  "Qaph")
ELEM (REH,                   "Reh")
ELEM (REVERSED_PE,           "Reversed Pe")
ELEM (SAD,                   "Sad")
ELEM (SADHE,                 "Sadhe")
ELEM (SEEN,                  "Seen")
ELEM (SEMKATH,               "Semkath")
ELEM (SHIN,                  "Shin")
ELEM (SWASH_KAF,             "Swash Kaf")
ELEM (SYRIAC_WAW,            "Syriac Waw")
ELEM (TAH,                   "Tah")
ELEM (TAW,                   "Taw")
ELEM (TEH_MARBUTA,           "Teh Marbuta")
ELEM (TEH_MARBUTA_GOAL,      "Teh Marbuta Goal")
ELEM (TETH,                  "Teth")
ELEM (WAW,                   "Waw")
ELEM (YEH,                   "Yeh")
ELEM (YEH_BARREE,            "Yeh Barree")
ELEM (YEH_WITH_TAIL,         "Yeh With Tail")
ELEM (YUDH,                  "Yudh")
ELEM (YUDH_HE,               "Yudh He")
ELEM (ZAIN,                  "Zain")
ELEM (ZHAIN,                 "Zhain")
