/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* Test the Unicode character type functions.
   Copyright (C) 2007 Free Software Foundation, Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#include "test-predicate-part1.h"

    { 0x0029, 0x0029 },
    { 0x005D, 0x005D },
    { 0x007D, 0x007D },
    { 0x0F3B, 0x0F3B },
    { 0x0F3D, 0x0F3D },
    { 0x169C, 0x169C },
    { 0x2046, 0x2046 },
    { 0x207E, 0x207E },
    { 0x208E, 0x208E },
    { 0x232A, 0x232A },
    { 0x2769, 0x2769 },
    { 0x276B, 0x276B },
    { 0x276D, 0x276D },
    { 0x276F, 0x276F },
    { 0x2771, 0x2771 },
    { 0x2773, 0x2773 },
    { 0x2775, 0x2775 },
    { 0x27C6, 0x27C6 },
    { 0x27E7, 0x27E7 },
    { 0x27E9, 0x27E9 },
    { 0x27EB, 0x27EB },
    { 0x27ED, 0x27ED },
    { 0x27EF, 0x27EF },
    { 0x2984, 0x2984 },
    { 0x2986, 0x2986 },
    { 0x2988, 0x2988 },
    { 0x298A, 0x298A },
    { 0x298C, 0x298C },
    { 0x298E, 0x298E },
    { 0x2990, 0x2990 },
    { 0x2992, 0x2992 },
    { 0x2994, 0x2994 },
    { 0x2996, 0x2996 },
    { 0x2998, 0x2998 },
    { 0x29D9, 0x29D9 },
    { 0x29DB, 0x29DB },
    { 0x29FD, 0x29FD },
    { 0x2E23, 0x2E23 },
    { 0x2E25, 0x2E25 },
    { 0x2E27, 0x2E27 },
    { 0x2E29, 0x2E29 },
    { 0x3009, 0x3009 },
    { 0x300B, 0x300B },
    { 0x300D, 0x300D },
    { 0x300F, 0x300F },
    { 0x3011, 0x3011 },
    { 0x3015, 0x3015 },
    { 0x3017, 0x3017 },
    { 0x3019, 0x3019 },
    { 0x301B, 0x301B },
    { 0x301E, 0x301F },
    { 0xFD3F, 0xFD3F },
    { 0xFE18, 0xFE18 },
    { 0xFE36, 0xFE36 },
    { 0xFE38, 0xFE38 },
    { 0xFE3A, 0xFE3A },
    { 0xFE3C, 0xFE3C },
    { 0xFE3E, 0xFE3E },
    { 0xFE40, 0xFE40 },
    { 0xFE42, 0xFE42 },
    { 0xFE44, 0xFE44 },
    { 0xFE48, 0xFE48 },
    { 0xFE5A, 0xFE5A },
    { 0xFE5C, 0xFE5C },
    { 0xFE5E, 0xFE5E },
    { 0xFF09, 0xFF09 },
    { 0xFF3D, 0xFF3D },
    { 0xFF5D, 0xFF5D },
    { 0xFF60, 0xFF60 },
    { 0xFF63, 0xFF63 }

#define PREDICATE(c) uc_is_general_category (c, UC_CATEGORY_Pe)
#include "test-predicate-part2.h"
