/* DO NOT EDIT! GENERATED AUTOMATICALLY! */
/* ISO C <ctype.h> like properties of Unicode characters.  */
/* Generated automatically by gen-uni-tables.c for Unicode 6.0.0.  */
#define header_0 16
#define header_2 9
#define header_3 127
#define header_4 15
static const
struct
  {
    int header[1];
    int level1[3];
    short level2[3 << 7];
    /*unsigned*/ int level3[51 << 4];
  }
u_is_alpha =
{
  { 3 },
  {
        4 * sizeof (int) / sizeof (short) +     0,
        4 * sizeof (int) / sizeof (short) +   128,
        4 * sizeof (int) / sizeof (short) +   256
  },
  {
        4 +   384 * sizeof (short) / sizeof (int) +     0,
        4 +   384 * sizeof (short) / sizeof (int) +    16,
        4 +   384 * sizeof (short) / sizeof (int) +    32,
        4 +   384 * sizeof (short) / sizeof (int) +    48,
        4 +   384 * sizeof (short) / sizeof (int) +    64,
        4 +   384 * sizeof (short) / sizeof (int) +    80,
        4 +   384 * sizeof (short) / sizeof (int) +    96,
        4 +   384 * sizeof (short) / sizeof (int) +   112,
        4 +   384 * sizeof (short) / sizeof (int) +   128,
        4 +   384 * sizeof (short) / sizeof (int) +   144,
        4 +   384 * sizeof (short) / sizeof (int) +   160,
        4 +   384 * sizeof (short) / sizeof (int) +   176,
        4 +   384 * sizeof (short) / sizeof (int) +   192,
        4 +   384 * sizeof (short) / sizeof (int) +   208,
        4 +   384 * sizeof (short) / sizeof (int) +   224,
        4 +   384 * sizeof (short) / sizeof (int) +   240,
        4 +   384 * sizeof (short) / sizeof (int) +   256,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   272,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   288,
        4 +   384 * sizeof (short) / sizeof (int) +   304,
        4 +   384 * sizeof (short) / sizeof (int) +   320,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   352,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   368,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   384,
        4 +   384 * sizeof (short) / sizeof (int) +   400,
        4 +   384 * sizeof (short) / sizeof (int) +   416,
        4 +   384 * sizeof (short) / sizeof (int) +   432,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   448,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   464,
        4 +   384 * sizeof (short) / sizeof (int) +   480,
        4 +   384 * sizeof (short) / sizeof (int) +   496,
        4 +   384 * sizeof (short) / sizeof (int) +   512,
        4 +   384 * sizeof (short) / sizeof (int) +   528,
        4 +   384 * sizeof (short) / sizeof (int) +   544,
        4 +   384 * sizeof (short) / sizeof (int) +   560,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   576,
        4 +   384 * sizeof (short) / sizeof (int) +   592,
        4 +   384 * sizeof (short) / sizeof (int) +   608,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   624,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   640,
        4 +   384 * sizeof (short) / sizeof (int) +   656,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   672,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   688,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   704,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   720,
        4 +   384 * sizeof (short) / sizeof (int) +   736,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   752,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   768,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   784,
        4 +   384 * sizeof (short) / sizeof (int) +   800,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
       -1,
        4 +   384 * sizeof (short) / sizeof (int) +   336,
        4 +   384 * sizeof (short) / sizeof (int) +   800,
       -1,
       -1
  },
  {
    0x00000000, 0x00000000, 0x07FFFFFE, 0x07FFFFFE,
    0x00000000, 0x04200400, 0xFF7FFFFF, 0xFF7FFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x0003FFC3, 0x0000501F,
    0x00000000, 0x00000000, 0x00000020, 0x3CDF0000,
    0xFFFFD740, 0xFFFFFFFB, 0xFFFFFFFF, 0xFFBFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFC03, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFE00FF, 0x027FFFFF, 0xFFFFFFFE,
    0x000000FF, 0x00000000, 0xFFFF0000, 0x000707FF,
    0x00000000, 0xFFFFFFFF, 0x000007FF, 0xFFFEC3FF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x002FFFFF, 0x9FFFC060,
    0xFFFD0000, 0x0000FFFF, 0xFFFFE000, 0xFFFFFFFF,
    0xFFFFFFFF, 0x0002003F, 0xFFFFFFFF, 0x043007FF,
    0x043FFFFF, 0x00000110, 0x01FFFFFF, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFF0, 0x23FFFFFF, 0xFF010000, 0xFEFEFFC3,
    0xFFF99FE0, 0x23C5FDFF, 0xB0004000, 0x0003FFC3,
    0xFFF987E0, 0x036DFDFF, 0x5E000000, 0x001CFFC0,
    0xFFFBBFE0, 0x23EDFDFF, 0x00010000, 0x0000FFC3,
    0xFFF99FE0, 0x23EDFDFF, 0xB0000000, 0x0002FFC3,
    0xD63DC7E8, 0x03FFC718, 0x00010000, 0x0000FFC0,
    0xFFFDDFE0, 0x23EFFDFF, 0x03000000, 0x0000FFC3,
    0xFFFDDFE0, 0x23EFFDFF, 0x40000000, 0x0006FFC3,
    0xFFFDDFE0, 0x27FFFFFF, 0x00004000, 0xFC00FFC3,
    0xFC7FFFE0, 0x2FFBFFFF, 0x0000007F, 0x00000000,
    0xFFFFFFFE, 0x07FF7FFF, 0x03FF7FBF, 0x00000000,
    0xFEF02596, 0x200DECAE, 0x33FF005F, 0x00000000,
    0x00000001, 0x000003FF, 0xFFFFFEFF, 0x00001FFF,
    0x00001F00, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0x800007FF, 0x3C3F03FF, 0xFFE1C062,
    0x03FF4003, 0xFFFFFFFF, 0xFFFF003F, 0x17FFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x3D7F3DFF, 0xFFFFFFFF,
    0xFFFF3DFF, 0x7F3DFFFF, 0xFF7FFF3D, 0xFFFFFFFF,
    0xFF3DFFFF, 0xFFFFFFFF, 0x07FFFFFF, 0x00000000,
    0x0000FFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x001FFFFF,
    0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFF9FFF,
    0x07FFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0x0001C7FF,
    0x0003DFFF, 0x0003FFFF, 0x0003FFFF, 0x0001DFFF,
    0xFFFFFFFF, 0x000FFFFF, 0x10800000, 0x000003FF,
    0x03FF0000, 0xFFFFFFFF, 0xFFFFFFFF, 0x00FFFFFF,
    0xFFFFFFFF, 0xFFFF05FF, 0xFFFFFFFF, 0x003FFFFF,
    0x1FFFFFFF, 0x00000000, 0xFFFFFFC0, 0x001F3FFF,
    0xFFFFFFFF, 0x00000FFF, 0x03FF00FE, 0x00000000,
    0x007FFFFF, 0xFFFFFFFF, 0x001FFFFF, 0x00000000,
    0x03FF03FF, 0x00000080, 0x00000000, 0x00000000,
    0xFFFFFFE0, 0x000FFFFF, 0x03FF0FE0, 0x00000000,
    0xFFFFFFF8, 0x03FFC001, 0xFFFFFFFF, 0x0000003F,
    0xFFFFFFFF, 0x0000000F, 0xFFFFE3FF, 0x3FFFFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x0003DE00,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x3F3FFFFF, 0xFFFFFFFF, 0xAAFF3F3F, 0x3FFFFFFF,
    0xFFFFFFFF, 0x5FDFFFFF, 0x0FCF1FDC, 0x1FDC1FFF,
    0x00000000, 0x00000000, 0x00000000, 0x80020000,
    0x1FFF0000, 0x00000000, 0x00000000, 0x00000000,
    0x3E2FFC84, 0xF3FFBF50, 0x000043E0, 0xFFFFFFFF,
    0x000001FF, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xF0000000, 0xFFFFFFFF, 0xFFFFFFFF, 0x000003FF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFF7FFF, 0x7FFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0000781F,
    0xFFFFFFFF, 0xFFFF003F, 0xFFFFFFFF, 0x0000803F,
    0x007FFFFF, 0x7F7F7F7F, 0x7F7F7F7F, 0x00000000,
    0x00000000, 0x00008000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x000000E0, 0x1F3E03FE, 0xFFFFFFFE, 0xFFFFFFFF,
    0xE07FFFFF, 0xFFFFFFFE, 0xFFFFFFFF, 0xF7FFFFFF,
    0xFFFFFFE0, 0xFFFE3FFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x00007FFF, 0x07FFFFFF, 0x00000000, 0xFFFF0000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x003FFFFF, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x00000FFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x00001FFF, 0x00000000, 0xFFFF0000, 0x3FFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFF1FFF, 0x00000FFF, 0xFFFFFFFF, 0x80007FFF,
    0x00FFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0000FFFF,
    0xFF800000, 0xFFFFFFFC, 0xFFFFFFFF, 0xFFFFFFFF,
    0x000379FF, 0x000003FF, 0x00000000, 0xFC000000,
    0xFFFFF7BB, 0x00000007, 0xFFFFFFFF, 0x000FFFFF,
    0xFFFFFFFC, 0x000FFFFF, 0x03FF0000, 0x08FC0000,
    0xFFFFFFFF, 0xFFFF003F, 0x0000007F, 0x1FFFFFFF,
    0xFFFFFFF0, 0x0007FFFF, 0x03FF8000, 0x00000000,
    0xFFFFFFFF, 0x000001FF, 0x03FF0FF7, 0x047FFFFF,
    0xFFFFFFFF, 0x3E62FFFF, 0x38000005, 0x00000000,
    0x007E7E7E, 0x00007F7F, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0xFFFFFFFF, 0x03FF0007,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFF000F, 0xFFFFF87F, 0x0FFFFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFF3FFF, 0xFFFFFFFF, 0xFFFF3FFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x03FFFFFF, 0x00000000,
    0xA0F8007F, 0x5F7FFDFF, 0xFFFFFFDB, 0xFFFFFFFF,
    0xFFFFFFFF, 0x0003FFFF, 0xFFF80000, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x3FFFFFFF, 0xFFFF0000, 0xFFFFFFFF,
    0xFFFCFFFF, 0xFFFFFFFF, 0x000000FF, 0x0FFF0000,
    0x00000000, 0x00000000, 0x00000000, 0xFFDF0000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x1FFFFFFF,
    0x03FF0000, 0x07FFFFFE, 0x07FFFFFE, 0xFFFFFFC0,
    0xFFFFFFFF, 0x7FFFFFFF, 0x1CFCFCFC, 0x00000000,
    0xFFFFEFFF, 0xB7FFFF7F, 0x3FFF3FFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x07FFFFFF,
    0x00000000, 0x00000000, 0xFFFFFFFF, 0x001FFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x1FFFFFFF, 0xFFFFFFFF, 0x0001FFFF, 0x00000000,
    0x7FFFFFFF, 0xFFFF0000, 0x000007FF, 0x00000000,
    0x3FFFFFFF, 0xFFFFFFFF, 0x003EFF0F, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x3FFFFFFF, 0x000003FF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFD3F, 0x91BFFFFF, 0x003FFFFF, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x003FFFFF, 0x03FFFFFF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFEEF0001, 0x000FFFFF, 0x00000000, 0x1FFFFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0x003FFFFF, 0x003FFFFF, 0x0007FFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0x000001FF, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFF8, 0x00FFFFFF, 0x00000000, 0x0000FFC0,
    0xFFFFFFF8, 0x0000FFFF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00007FFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000007,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0x00007FFF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0x01FFFFFF, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000003, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFDFFFFF, 0xFFFFFFFF,
    0xDFFFFFFF, 0xEBFFDE64, 0xFFFFFFEF, 0xFFFFFFFF,
    0xDFDFE7BF, 0x7BFFFFFF, 0xFFFDFC5F, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFF3F, 0xF7FFFFFD, 0xF7FFFFFF,
    0xFFDFFFFF, 0xFFDFFFFF, 0xFFFF7FFF, 0xFFFF7FFF,
    0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFCFF7, 0xFFFFFFFF,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0xFFFF0000, 0xFFFF1FFF, 0xFFFF03FF, 0xFFFF03FF,
    0x000007FF, 0x00000000, 0x00000000, 0xFFFFFFC0,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0x007FFFFF, 0x00000000,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0x001FFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
    0x3FFFFFFF, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000,
    0x00000000, 0x00000000, 0x00000000, 0x00000000
  }
};
